@@ -188,6 +188,8 @@ public class WSSecSignature extends WSSecSignatureBase {
         if (!useCustomSecRef) {
             secRef = new SecurityTokenReference(doc);
             strUri = getWsConfig().getIdAllocator().createSecureId("STR-", secRef);
+            secRef.addWSSENamespace();
+            secRef.addWSUNamespace();
             secRef.setID(strUri);
             
             //
