@@ -31,6 +31,7 @@ import org.apache.ws.security.util.WSSecurityUtil;
 import org.apache.ws.security.util.WSTimeSource;
 import org.apache.ws.security.util.XmlSchemaDateFormat;
 import org.apache.ws.security.util.Base64;
+import org.w3c.dom.CDATASection;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
@@ -190,7 +191,7 @@ public class UsernameToken {
         }
         
         if (elementPassword != null) {
-            if (elementPassword.hasAttribute(WSConstants.PASSWORD_TYPE_ATTR)) {
+            if (elementPassword.hasAttributeNS(null, WSConstants.PASSWORD_TYPE_ATTR)) {
                 passwordType = elementPassword.getAttributeNS(null, WSConstants.PASSWORD_TYPE_ATTR);
             } else if (elementPassword.hasAttributeNS(
                 WSConstants.WSSE_NS, WSConstants.PASSWORD_TYPE_ATTR)
@@ -664,6 +665,9 @@ public class UsernameToken {
                 if (Node.TEXT_NODE == node.getNodeType()) {
                     found = true;
                     builder.append(((Text)node).getData());
+                } else if (Node.CDATA_SECTION_NODE == node.getNodeType()) {
+                    found = true;
+                    builder.append(((CDATASection)node).getData());
                 }
                 node = node.getNextSibling();
             }
