@@ -80,7 +80,9 @@ public class BinarySecurityTokenProcessor implements Processor {
         WSSecurityEngineResult result = 
             new WSSecurityEngineResult(WSConstants.BST, token, certs);
         wsDocInfo.addTokenElement(elem);
-        result.put(WSSecurityEngineResult.TAG_ID, id);
+        if (!"".equals(id)) {
+            result.put(WSSecurityEngineResult.TAG_ID, id);
+        }
         
         if (validator != null) {
             // Hook to allow the user to validate the BinarySecurityToken
@@ -110,6 +112,11 @@ public class BinarySecurityTokenProcessor implements Processor {
                 result.put(WSSecurityEngineResult.TAG_PRINCIPAL, certs[0].getSubjectX500Principal());
             }
             result.put(WSSecurityEngineResult.TAG_SUBJECT, credential.getSubject());
+            
+            if (credential.getDelegationCredential() != null) {
+                result.put(WSSecurityEngineResult.TAG_DELEGATION_CREDENTIAL, 
+                           credential.getDelegationCredential());
+            }
         }
         
         wsDocInfo.addResult(result);
