@@ -19,6 +19,7 @@
 
 package org.apache.ws.security.processor;
 
+import java.security.NoSuchAlgorithmException;
 import java.security.PrivateKey;
 import java.security.cert.X509Certificate;
 import java.security.spec.MGF1ParameterSpec;
@@ -189,7 +190,10 @@ public class EncryptedKeyProcessor implements Processor {
             WSSecurityEngineResult.TAG_ENCRYPTED_KEY_TRANSPORT_METHOD, 
             encryptedKeyTransportMethod
         );
-        result.put(WSSecurityEngineResult.TAG_ID, elem.getAttributeNS(null, "Id"));
+        String tokenId = elem.getAttributeNS(null, "Id");
+        if (!"".equals(tokenId)) {
+            result.put(WSSecurityEngineResult.TAG_ID, tokenId);
+        }
         result.put(WSSecurityEngineResult.TAG_X509_REFERENCE_TYPE, strParser.getCertificatesReferenceType());
         wsDocInfo.addResult(result);
         wsDocInfo.addTokenElement(elem);
@@ -209,7 +213,7 @@ public class EncryptedKeyProcessor implements Processor {
     private static byte[] getRandomKey(List<String> dataRefURIs, Document doc, WSDocInfo wsDocInfo) throws WSSecurityException {
         try {
             String alg = "AES";
-            int size = 128;
+            int size = 16;
             if (!dataRefURIs.isEmpty()) {
                 String uri = dataRefURIs.iterator().next();
                 Element ee = ReferenceListProcessor.findEncryptedDataElement(doc, wsDocInfo, uri);
@@ -221,8 +225,16 @@ public class EncryptedKeyProcessor implements Processor {
             kgen.init(size * 8);
             SecretKey k = kgen.generateKey();
             return k.getEncoded();
-        } catch (Exception ex) {
-            throw new WSSecurityException(WSSecurityException.FAILED_CHECK, null, null, ex);
+        } catch (Throwable ex) {
+            // Fallback to just using AES to avoid attacks on EncryptedData algorithms
+            try {
+                KeyGenerator kgen = KeyGenerator.getInstance("AES");
+                kgen.init(128);
+                SecretKey k = kgen.generateKey();
+                return k.getEncoded();
+            } catch (NoSuchAlgorithmException e) {
+                throw new WSSecurityException(WSSecurityException.FAILED_CHECK, null, null, e);
+            }
         }
     }
     
@@ -403,7 +415,7 @@ public class EncryptedKeyProcessor implements Processor {
         Element encryptedDataElement = 
             ReferenceListProcessor.findEncryptedDataElement(doc, docInfo, dataRefURI);
         if (encryptedDataElement != null && data.isRequireSignedEncryptedDataElements()) {
-            WSSecurityUtil.verifySignedElement(encryptedDataElement, doc, docInfo.getSecurityHeader());
+            WSSecurityUtil.verifySignedElement(encryptedDataElement, docInfo);
         }
         //
         // Prepare the SecretKey object to decrypt EncryptedData
