@@ -127,6 +127,7 @@ public class SignatureProcessor implements Processor {
         String signatureMethod = getSignatureMethod(elem);
         REFERENCE_TYPE referenceType = null;
 
+        Credential credential = new Credential();
         Validator validator = data.getValidator(WSSecurityEngine.SIGNATURE);
         if (keyInfoElement == null) {
             certs = getDefaultCerts(data.getSigCrypto());
@@ -153,11 +154,10 @@ public class SignatureProcessor implements Processor {
             if (strElements.isEmpty()) {
                 publicKey = parseKeyValue(keyInfoElement);
                 if (validator != null) {
-                    Credential credential = new Credential();
                     credential.setPublicKey(publicKey);
                     principal = new PublicKeyPrincipal(publicKey);
                     credential.setPrincipal(principal);
-                    validator.validate(credential, data);
+                    credential = validator.validate(credential, data);
                 }
             } else {
                 STRParser strParser = new SignatureSTRParser();
@@ -180,11 +180,10 @@ public class SignatureProcessor implements Processor {
                     LOG.debug("Direct Trust for SAML/BST credential");
                 }
                 if (!trusted && (publicKey != null || certs != null) && (validator != null)) {
-                    Credential credential = new Credential();
                     credential.setPublicKey(publicKey);
                     credential.setCertificates(certs);
                     credential.setPrincipal(principal);
-                    validator.validate(credential, data);
+                    credential = validator.validate(credential, data);
                 }
             }
         }
@@ -251,12 +250,17 @@ public class SignatureProcessor implements Processor {
                 certs, dataRefs, signatureValue);
         result.put(WSSecurityEngineResult.TAG_SIGNATURE_METHOD, signatureMethod);
         result.put(WSSecurityEngineResult.TAG_CANONICALIZATION_METHOD, c14nMethod);
-        result.put(WSSecurityEngineResult.TAG_ID, elem.getAttributeNS(null, "Id"));
+        String tokenId = elem.getAttributeNS(null, "Id");
+        if (!"".equals(tokenId)) {
+            result.put(WSSecurityEngineResult.TAG_ID, tokenId);
+        }
         result.put(WSSecurityEngineResult.TAG_SECRET, secretKey);
         result.put(WSSecurityEngineResult.TAG_PUBLIC_KEY, publicKey);
         result.put(WSSecurityEngineResult.TAG_X509_REFERENCE_TYPE, referenceType);
+        result.put(WSSecurityEngineResult.TAG_TOKEN_ELEMENT, elem);
         if (validator != null) {
             result.put(WSSecurityEngineResult.TAG_VALIDATED_TOKEN, Boolean.TRUE);
+            result.put(WSSecurityEngineResult.TAG_SUBJECT, credential.getSubject());
         }
         wsDocInfo.addResult(result);
         wsDocInfo.addTokenElement(elem);
@@ -414,7 +418,7 @@ public class SignatureProcessor implements Processor {
             }
             
             // Test for replay attacks
-            testMessageReplay(elem, xmlSignature.getSignatureValue().getValue(), data, wsDocInfo);
+            testMessageReplay(elem, xmlSignature.getSignatureValue().getValue(), key, data, wsDocInfo);
             
             setElementsOnContext(xmlSignature, (DOMValidateContext)context, wsDocInfo, elem.getOwnerDocument());
             boolean signatureOk = xmlSignature.validate(context);
@@ -627,9 +631,11 @@ public class SignatureProcessor implements Processor {
     }
     
     /**
-     * Test for a replayed message. The cache key is the Timestamp Created String and the signature value.
+     * Test for a replayed message. The cache key is the Timestamp Created String, the signature 
+     * value, and the encoded value of the signing key.
      * @param signatureElement
      * @param signatureValue
+     * @param key
      * @param requestData
      * @param wsDocInfo
      * @throws WSSecurityException
@@ -637,6 +643,7 @@ public class SignatureProcessor implements Processor {
     private void testMessageReplay(
         Element signatureElement,
         byte[] signatureValue,
+        Key key,
         RequestData requestData,
         WSDocInfo wsDocInfo
     ) throws WSSecurityException {
@@ -670,7 +677,8 @@ public class SignatureProcessor implements Processor {
         // Test for replay attacks
         Date created = timeStamp.getCreated();
         DateFormat zulu = new XmlSchemaDateFormat();
-        String identifier = zulu.format(created) + "" + Arrays.hashCode(signatureValue);
+        String identifier = zulu.format(created) + "" + Arrays.hashCode(signatureValue)
+            + "" + Arrays.hashCode(key.getEncoded());
 
         if (replayCache.contains(identifier)) {
             throw new WSSecurityException(
@@ -680,7 +688,7 @@ public class SignatureProcessor implements Processor {
             );
         }
 
-        // Store the Timestamp/SignatureValue combination in the cache
+        // Store the Timestamp/SignatureValue/Key combination in the cache
         Date expires = timeStamp.getExpires();
         if (expires != null) {
             Date rightNow = new Date();
