@@ -217,13 +217,22 @@ public class AssertionWrapper {
      * @param parms of type SAMLParms
      */
     public AssertionWrapper(SAMLParms parms) throws WSSecurityException {
+        if (parms == null || parms.getCallbackHandler() == null) {
+            throw new IllegalStateException(
+               "No SAML CallbackHandler has been configured"
+            );
+        }
+        
         OpenSAMLUtil.initSamlEngine();
         
         //
         // Create the SAML callback that the handler will use to get the required data from the 
         // client application.
         //
-        SAMLCallback[] samlCallbacks = new SAMLCallback[] { new SAMLCallback() };
+        SAMLCallback samlCallback = new SAMLCallback();
+        samlCallback.setSamlVersion(parms.getSAMLVersion());
+        
+        SAMLCallback[] samlCallbacks = new SAMLCallback[] { samlCallback };
 
         try {
             // Get the SAML source data using the currently configured callback implementation.
