@@ -23,20 +23,20 @@ import org.apache.ws.security.WSSecurityException;
 import org.apache.ws.security.saml.ext.bean.ActionBean;
 import org.apache.ws.security.saml.ext.bean.AttributeBean;
 import org.apache.ws.security.saml.ext.bean.AttributeStatementBean;
+import org.apache.ws.security.saml.ext.bean.AudienceRestrictionBean;
 import org.apache.ws.security.saml.ext.bean.AuthDecisionStatementBean;
 import org.apache.ws.security.saml.ext.bean.AuthenticationStatementBean;
 import org.apache.ws.security.saml.ext.bean.ConditionsBean;
 import org.apache.ws.security.saml.ext.bean.KeyInfoBean;
+import org.apache.ws.security.saml.ext.bean.ProxyRestrictionBean;
 import org.apache.ws.security.saml.ext.bean.SubjectBean;
 import org.apache.ws.security.saml.ext.bean.SubjectConfirmationDataBean;
 import org.apache.ws.security.saml.ext.bean.SubjectLocalityBean;
 import org.apache.ws.security.util.UUIDGenerator;
-
 import org.joda.time.DateTime;
 import org.opensaml.Configuration;
 import org.opensaml.common.SAMLObjectBuilder;
 import org.opensaml.common.SAMLVersion;
-
 import org.opensaml.saml2.core.Action;
 import org.opensaml.saml2.core.Assertion;
 import org.opensaml.saml2.core.Attribute;
@@ -54,11 +54,12 @@ import org.opensaml.saml2.core.Evidence;
 import org.opensaml.saml2.core.Issuer;
 import org.opensaml.saml2.core.KeyInfoConfirmationDataType;
 import org.opensaml.saml2.core.NameID;
+import org.opensaml.saml2.core.OneTimeUse;
+import org.opensaml.saml2.core.ProxyRestriction;
 import org.opensaml.saml2.core.Subject;
 import org.opensaml.saml2.core.SubjectConfirmation;
 import org.opensaml.saml2.core.SubjectConfirmationData;
 import org.opensaml.saml2.core.SubjectLocality;
-
 import org.opensaml.xml.XMLObject;
 import org.opensaml.xml.XMLObjectBuilderFactory;
 import org.opensaml.xml.schema.XSString;
@@ -66,9 +67,9 @@ import org.opensaml.xml.schema.impl.XSStringBuilder;
 import org.opensaml.xml.signature.KeyInfo;
 
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.List;
 
-
 /**
  * Class SAML2ComponentBuilder provides builder methods that can be used
  * to construct SAML v2.0 statements using the OpenSaml library.
@@ -86,6 +87,10 @@ public final class SAML2ComponentBuilder {
     
     private static volatile SAMLObjectBuilder<SubjectConfirmation> subjectConfirmationBuilder;
     
+    private static volatile SAMLObjectBuilder<OneTimeUse> oneTimeUseBuilder;
+    
+    private static volatile SAMLObjectBuilder<ProxyRestriction> proxyRestrictionBuilder;
+    
     private static volatile SAMLObjectBuilder<Conditions> conditionsBuilder;
     
     private static volatile SAMLObjectBuilder<SubjectConfirmationData> subjectConfirmationDataBuilder;
@@ -172,7 +177,7 @@ public final class SAML2ComponentBuilder {
      * @param conditionsBean A ConditionsBean object
      * @return a Conditions object
      */
-    @SuppressWarnings("unchecked")
+    @SuppressWarnings({ "unchecked", "deprecation" })
     public static Conditions createConditions(ConditionsBean conditionsBean) {
         if (conditionsBuilder == null) {
             conditionsBuilder = (SAMLObjectBuilder<Conditions>) 
@@ -215,6 +220,23 @@ public final class SAML2ComponentBuilder {
             conditions.getAudienceRestrictions().add(audienceRestriction);
         }
         
+        if (conditionsBean.getAudienceRestrictions() != null 
+            && !conditionsBean.getAudienceRestrictions().isEmpty()) {
+            for (AudienceRestrictionBean audienceRestrictionBean 
+                : conditionsBean.getAudienceRestrictions()) {
+                AudienceRestriction audienceRestriction = 
+                        createAudienceRestriction(audienceRestrictionBean);
+                conditions.getAudienceRestrictions().add(audienceRestriction);
+            }
+        }
+        
+        if (conditionsBean.isOneTimeUse()) {
+            conditions.getConditions().add(createOneTimeUse());
+        }
+        
+        if (conditionsBean.getProxyRestriction() != null) {
+            conditions.getConditions().add(createProxyRestriction(conditionsBean.getProxyRestriction()));
+        }
         return conditions;
     }
 
@@ -224,8 +246,23 @@ public final class SAML2ComponentBuilder {
      * @param audienceURI of type String
      * @return an AudienceRestriction object
      */
-    @SuppressWarnings("unchecked")
+    @Deprecated
     public static AudienceRestriction createAudienceRestriction(String audienceURI) {
+        AudienceRestrictionBean audienceRestrictionBean = new AudienceRestrictionBean();
+        audienceRestrictionBean.setAudienceURIs(Collections.singletonList(audienceURI));
+        return createAudienceRestriction(audienceRestrictionBean);
+    }
+    
+    /**
+     * Create an AudienceRestriction object
+     *
+     * @param audienceRestrictionBean of type AudienceRestrictionBean
+     * @return an AudienceRestriction object
+     */
+    @SuppressWarnings("unchecked")
+    public static AudienceRestriction createAudienceRestriction(
+        AudienceRestrictionBean audienceRestrictionBean
+    ) {
         if (audienceRestrictionBuilder == null) {
             audienceRestrictionBuilder = (SAMLObjectBuilder<AudienceRestriction>) 
                 builderFactory.getBuilder(AudienceRestriction.DEFAULT_ELEMENT_NAME);
@@ -236,11 +273,61 @@ public final class SAML2ComponentBuilder {
         }
        
         AudienceRestriction audienceRestriction = audienceRestrictionBuilder.buildObject();
-        Audience audience = audienceBuilder.buildObject();
-        audience.setAudienceURI(audienceURI);
-        audienceRestriction.getAudiences().add(audience);
+        
+        for (String audienceURI : audienceRestrictionBean.getAudienceURIs()) {
+            Audience audience = audienceBuilder.buildObject();
+            audience.setAudienceURI(audienceURI);
+            audienceRestriction.getAudiences().add(audience);
+        }
         return audienceRestriction;
     }
+    
+    /**
+     * Create a OneTimeUse object
+     *
+     * @return a OneTimeUse object
+     */
+    @SuppressWarnings("unchecked")
+    public static OneTimeUse createOneTimeUse() {
+        if (oneTimeUseBuilder == null) {
+            oneTimeUseBuilder = (SAMLObjectBuilder<OneTimeUse>) 
+                builderFactory.getBuilder(OneTimeUse.DEFAULT_ELEMENT_NAME);
+        }
+       
+        return oneTimeUseBuilder.buildObject();
+    }
+    
+    /**
+     * Create a ProxyRestriction object
+     *
+     * @return a ProxyRestriction object
+     */
+    @SuppressWarnings("unchecked")
+    public static ProxyRestriction createProxyRestriction(ProxyRestrictionBean proxyRestrictionBean) {
+        if (proxyRestrictionBuilder == null) {
+            proxyRestrictionBuilder = (SAMLObjectBuilder<ProxyRestriction>) 
+                builderFactory.getBuilder(ProxyRestriction.DEFAULT_ELEMENT_NAME);
+        }
+        
+        ProxyRestriction proxyRestriction = proxyRestrictionBuilder.buildObject();
+        if (proxyRestrictionBean.getCount() > 0) {
+            proxyRestriction.setProxyCount(proxyRestrictionBean.getCount());
+        }
+        
+        if (!proxyRestrictionBean.getAudienceURIs().isEmpty()) {
+            if (audienceBuilder == null) {
+                audienceBuilder = (SAMLObjectBuilder<Audience>) 
+                    builderFactory.getBuilder(Audience.DEFAULT_ELEMENT_NAME);
+            }
+            for (String audienceURI : proxyRestrictionBean.getAudienceURIs()) {
+                Audience audience = audienceBuilder.buildObject();
+                audience.setAudienceURI(audienceURI);
+                proxyRestriction.getAudiences().add(audience);
+            }
+        }
+        
+        return proxyRestriction;
+    }
 
     /**
      * Create SAML 2 Authentication Statement(s).
