@@ -24,6 +24,7 @@ import org.apache.ws.security.SOAP12Constants;
 import org.apache.ws.security.SOAPConstants;
 import org.apache.ws.security.WSConstants;
 import org.apache.ws.security.WSDataRef;
+import org.apache.ws.security.WSDocInfo;
 import org.apache.ws.security.WSEncryptionPart;
 import org.apache.ws.security.WSSecurityEngineResult;
 import org.apache.ws.security.WSSecurityException;
@@ -48,12 +49,11 @@ import javax.xml.namespace.QName;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.SecureRandom;
+import java.security.NoSuchProviderException;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Set;
 
 /**
  * WS-Security Utility methods. <p/>
@@ -410,8 +410,8 @@ public final class WSSecurityUtil {
             // start node processing at this point
             if (startNode.getNodeType() == Node.ELEMENT_NODE) {
                 Element se = (Element) startNode;
-                if ((se.hasAttribute("ID") && value.equals(se.getAttributeNS(null, "ID")))
-                    || (se.hasAttribute("AssertionID") 
+                if ((se.hasAttributeNS(null, "ID") && value.equals(se.getAttributeNS(null, "ID")))
+                    || (se.hasAttributeNS(null, "AssertionID") 
                         && value.equals(se.getAttributeNS(null, "AssertionID")))) {
                     if (foundElement == null) {
                         foundElement = se; // Continue searching to find duplicates
@@ -641,6 +641,9 @@ public final class WSSecurityUtil {
      *         correctly formed
      */
     public static String getIDFromReference(String ref) {
+        if (ref == null) {
+            return null;
+        }
         String id = ref.trim();
         if (id.length() == 0) {
             return null;
@@ -843,7 +846,12 @@ public final class WSSecurityUtil {
         throws WSSecurityException {
         try {
             String keyAlgorithm = JCEMapper.translateURItoJCEID(cipherAlgo);
-            return Cipher.getInstance(keyAlgorithm);
+            String provider = JCEMapper.getProviderId();
+            
+            if (provider == null) {
+                return Cipher.getInstance(keyAlgorithm);
+            }
+            return Cipher.getInstance(keyAlgorithm, provider);
         } catch (NoSuchPaddingException ex) {
             throw new WSSecurityException(
                 WSSecurityException.UNSUPPORTED_ALGORITHM, "unsupportedKeyTransp", 
@@ -867,6 +875,11 @@ public final class WSSecurityUtil {
                     new Object[] { "No such algorithm: " + cipherAlgo }, ex
                 );
             }
+        } catch (NoSuchProviderException ex) {
+            throw new WSSecurityException(
+                WSSecurityException.UNSUPPORTED_ALGORITHM, "unsupportedKeyTransp",
+                new Object[] { "No such provider " + JCEMapper.getProviderId() + " for: "  + cipherAlgo }, ex
+            );
         }
     }
     
@@ -1298,7 +1311,7 @@ public final class WSSecurityUtil {
         Element element
     ) {
         String id = uri;
-        if (uri.charAt(0) == '#') {
+        if (!"".equals(uri) && uri.charAt(0) == '#') {
             id = id.substring(1);
         }
         
@@ -1308,7 +1321,7 @@ public final class WSSecurityUtil {
         }
         if (element.hasAttributeNS(null, "Id")
             && id.equals(element.getAttributeNS(null, "Id"))) {
-    	    context.setIdAttributeNS(element, null, "Id");
+            context.setIdAttributeNS(element, null, "Id");
         }
         if (element.hasAttributeNS(null, "id")
             && id.equals(element.getAttributeNS(null, "id"))) {
@@ -1350,56 +1363,44 @@ public final class WSSecurityUtil {
         }
     }
     
-    public static void verifySignedElement(Element elem, Document doc, Element securityHeader)
+    public static void verifySignedElement(Element elem, WSDocInfo wsDocInfo)
         throws WSSecurityException {
-        final Element envelope = doc.getDocumentElement();
-        final Set<String> signatureRefIDs = getSignatureReferenceIDs(securityHeader);
-        if (!signatureRefIDs.isEmpty()) {
-            Node cur = elem;
-            while (!cur.isSameNode(envelope)) {
-                if (cur.getNodeType() == Node.ELEMENT_NODE) {
-                    if (WSConstants.SIG_LN.equals(cur.getLocalName())
-                        && WSConstants.SIG_NS.equals(cur.getNamespaceURI())) {
-                        throw new WSSecurityException(WSSecurityException.FAILED_CHECK,
-                            "requiredElementNotSigned", new Object[] {elem});
-                    } else if (isLinkedBySignatureRefs((Element)cur, signatureRefIDs)) {
-                        return;
+        List<WSSecurityEngineResult> signedResults = 
+            wsDocInfo.getResultsByTag(WSConstants.SIGN);
+        if (signedResults != null) {
+            for (WSSecurityEngineResult signedResult : signedResults) {
+                @SuppressWarnings("unchecked")
+                List<WSDataRef> dataRefs = 
+                    (List<WSDataRef>)signedResult.get(WSSecurityEngineResult.TAG_DATA_REF_URIS);
+                if (dataRefs != null) {
+                    for (WSDataRef dataRef : dataRefs) {
+                        if (isElementOrAncestorSigned(elem, dataRef.getProtectedElement())) {
+                            return;
+                        }
                     }
                 }
-                cur = cur.getParentNode();
             }
         }
+        
         throw new WSSecurityException(
             WSSecurityException.FAILED_CHECK, "requiredElementNotSigned", new Object[] {elem});
     }
     
-    private static boolean isLinkedBySignatureRefs(Element elem, Set<String> allIDs) {
-        // Try the wsu:Id first
-        String attributeNS = elem.getAttributeNS(WSConstants.WSU_NS, "Id");
-        if (!"".equals(attributeNS) && allIDs.contains(attributeNS)) {
-            return true;
-        }
-        attributeNS = elem.getAttributeNS(null, "Id");
-        return (!"".equals(attributeNS) && allIDs.contains(attributeNS));
-    }
-    
-    private static Set<String> getSignatureReferenceIDs(Element wsseHeader) throws WSSecurityException {
-        final Set<String> refs = new HashSet<String>();
-        final List<Element> signatures = WSSecurityUtil.getDirectChildElements(wsseHeader, WSConstants.SIG_LN, WSConstants.SIG_NS);
-        for (Element signature : signatures) {
-            Element sigInfo = WSSecurityUtil.getDirectChildElement(signature, WSConstants.SIG_INFO_LN, WSConstants.SIG_NS);
-            List<Element> references = WSSecurityUtil.getDirectChildElements(sigInfo, WSConstants.REF_LN, WSConstants.SIG_NS);
-            for (Element reference : references) {
-                String uri = reference.getAttributeNS(null, "URI");
-                if (!"".equals(uri)) {
-                    boolean added = refs.add(WSSecurityUtil.getIDFromReference(uri));
-                    if (!added) {
-                        log.warn("Duplicated reference uri: " + uri);
-                    }
-                }
+    /**
+     * Does the current element or some ancestor of it correspond to the known "signedElement"? 
+     */
+    private static boolean isElementOrAncestorSigned(Element elem, Element signedElement) 
+        throws WSSecurityException {
+        final Element envelope = elem.getOwnerDocument().getDocumentElement();
+        Node cur = elem;
+        while (!cur.isSameNode(envelope)) {
+            if (cur.getNodeType() == Node.ELEMENT_NODE && cur.equals(signedElement)) {
+                return true;
             }
+            cur = cur.getParentNode();
         }
-        return refs;
+        
+        return false;
     }
     
 }
