@@ -46,6 +46,9 @@ import java.util.List;
 import java.util.ArrayList;
 
 import javax.security.auth.callback.CallbackHandler;
+import javax.xml.datatype.DatatypeFactory;
+import javax.xml.datatype.Duration;
+import javax.xml.datatype.XMLGregorianCalendar;
 
 /**
  * This class tests the modification of requests.
@@ -477,7 +480,15 @@ public class ModifiedRequestTest extends org.junit.Assert {
         Element createdValue = 
             WSSecurityUtil.findElement(timestampElement, "Created", WSConstants.WSU_NS);
         DateFormat zulu = new XmlSchemaDateFormat();
-        createdValue.setTextContent(zulu.format(new Date()));
+        
+        DatatypeFactory datatypeFactory = DatatypeFactory.newInstance();
+        XMLGregorianCalendar createdCalendar = 
+            datatypeFactory.newXMLGregorianCalendar(createdValue.getTextContent());
+        // Add 5 seconds
+        Duration duration = datatypeFactory.newDuration(5000L);
+        createdCalendar.add(duration);
+        Date createdDate = createdCalendar.toGregorianCalendar().getTime();
+        createdValue.setTextContent(zulu.format(createdDate));
         
         if (LOG.isDebugEnabled()) {
             String outputString = 
