@@ -305,11 +305,7 @@ public class SignatureTest extends org.junit.Assert {
             LOG.debug(outputString);
         }
         
-        // Turn off BSP spec compliance
         WSSecurityEngine newEngine = new WSSecurityEngine();
-        WSSConfig config = WSSConfig.getNewInstance();
-        config.setWsiBSPCompliant(false);
-        newEngine.setWssConfig(config);
         List<WSSecurityEngineResult> results = newEngine.processSecurityHeader(doc, null, null, crypto);
         
         WSSecurityEngineResult actionResult =
@@ -319,16 +315,6 @@ public class SignatureTest extends org.junit.Assert {
         REFERENCE_TYPE referenceType = 
             (REFERENCE_TYPE)actionResult.get(WSSecurityEngineResult.TAG_X509_REFERENCE_TYPE);
         assertTrue(referenceType == REFERENCE_TYPE.KEY_IDENTIFIER);
-        
-        // Now turn on BSP spec compliance
-        config.setWsiBSPCompliant(true);
-        newEngine.setWssConfig(config);
-        try {
-            newEngine.processSecurityHeader(doc, null, null, crypto);
-            fail("Failure expected on a bad ValueType attribute");
-        } catch (WSSecurityException ex) {
-            // expected
-        }
     }
     
     /**
@@ -849,6 +835,31 @@ public class SignatureTest extends org.junit.Assert {
             newSecEngine.processSecurityHeader(elem, data);
         assertTrue(handler.checkResults(results, actions));
     }
+    
+    @org.junit.Test
+    public void testCommentInSOAPBody() throws Exception {
+        WSSecSignature builder = new WSSecSignature();
+        builder.setUserInfo("16c73ab6-b892-458f-abf5-2f875f74882e", "security");
+        LOG.info("Before Signing....");
+        Document doc = SOAPUtil.toSOAPPart(SOAPUtil.SAMPLE_SOAP_MSG);
+        WSSecHeader secHeader = new WSSecHeader();
+        secHeader.insertSecurityHeader(doc);
+        Document signedDoc = builder.build(doc, crypto, secHeader);
+        
+        // Add a comment node
+        Element body = WSSecurityUtil.findBodyElement(signedDoc);
+        org.w3c.dom.Node commentNode = signedDoc.createComment("This is a comment");
+        body.getFirstChild().appendChild(commentNode);
+
+        if (LOG.isDebugEnabled()) {
+            String outputString = 
+               org.apache.ws.security.util.XMLUtils.PrettyDocumentToString(signedDoc);
+            LOG.debug(outputString);
+        }
+        
+        verify(signedDoc);
+    }
+    
 
     /**
      * Verifies the soap envelope.
