@@ -38,6 +38,7 @@ import org.w3c.dom.Element;
 import java.text.DateFormat;
 import java.util.Date;
 import java.util.List;
+import java.util.Locale;
 
 /**
  * WS-Security Test Case for Timestamps.
@@ -770,6 +771,33 @@ public class TimestampTest extends org.junit.Assert {
         }
     }
     
+    @org.junit.Test
+    public void testThaiLocaleVerification() throws Exception {
+        
+        Locale defaultLocale = Locale.getDefault();
+        try {
+            Locale.setDefault(new Locale("th", "TH"));
+        
+            Document doc = SOAPUtil.toSOAPPart(SOAPUtil.SAMPLE_SOAP_MSG);
+            WSSecHeader secHeader = new WSSecHeader();
+            secHeader.insertSecurityHeader(doc);
+            
+            WSSecTimestamp timestamp = new WSSecTimestamp();
+            timestamp.setTimeToLive(300);
+            Document createdDoc = timestamp.build(doc, secHeader);
+            
+            //
+            // Do some processing
+            //
+            List<WSSecurityEngineResult> wsResult = verify(createdDoc, WSSConfig.getNewInstance());
+            WSSecurityEngineResult actionResult = 
+                WSSecurityUtil.fetchActionResult(wsResult, WSConstants.TS);
+            assertTrue(actionResult != null);
+        } finally {
+            Locale.setDefault(defaultLocale);
+        }
+    }
+    
     /**
      * Verifies the soap envelope
      * 
